package com.zhili.zkstationserver.handler;

import com.alibaba.fastjson.JSON;
import com.zhili.zkstationserver.configuration.ServerConfig;
import com.zhili.zkstationserver.dto.ChargerState;
import com.zhili.zkstationserver.dto.ExchangeEvent;
import com.zhili.zkstationserver.dto.ExchangeRecordInfo;
import com.zhili.zkstationserver.dto.ExchangeStartReply;
import com.zhili.zkstationserver.dto.StationConnInfo;
import com.zhili.zkstationserver.dto.StationMessage;
import com.zhili.zkstationserver.dto.StationMessage.CommandType;
import com.zhili.zkstationserver.util.ApplicationContextUtils;
import com.zhili.zkstationserver.util.BytesUtil;
import com.zhili.zkstationserver.util.StationCenter;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.kafka.core.KafkaTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/12/20 15:40
 */
@Slf4j
public class StationMessageHandler extends ChannelInboundHandlerAdapter {
    KafkaTemplate kafkaTemplate = (KafkaTemplate) ApplicationContextUtils.getApplicationContext().getBean("kafkaTemplate");
    ServerConfig serverConfig = ApplicationContextUtils.getApplicationContext().getBean(ServerConfig.class);
    RedisTemplate redisTemplate = (RedisTemplate) ApplicationContextUtils.getApplicationContext().getBean("redisTemplate");
    RedisScript stationRegisterScript = (DefaultRedisScript) ApplicationContextUtils.getApplicationContext().getBean("stationRegisterScript");

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("channel connected: " + ctx.channel().remoteAddress());
        StationCenter.add(ctx.channel());
        super.channelActive(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.info("channel disconnected: " + ctx.channel().remoteAddress());
        super.channelInactive(ctx);
        //断连后及时删除站的注册信息
        Object stationCodeObj = ctx.channel().attr(AttributeKey.valueOf("station")).get();
        if (stationCodeObj == null || StringUtils.isBlank(stationCodeObj.toString())) {
            return;
        }
        redisTemplate.delete("StationConn:" + stationCodeObj);
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        if (!(msg instanceof StationMessage)) {
            return;
        }
        StationMessage stationMessage = (StationMessage) msg;
        int command = stationMessage.getCommand();
        byte[] data = stationMessage.getData();
        //log.info("command:" + command + ",data:" + ArrayUtils.toString(data));
        String stationCode;
        StationMessage stationMessageReply;
        byte[] replyBytes;
        String topic;
        StationConnInfo stationConnInfo;
        List<String> stationRegisterkeys;
        Long registerSuccess;
        switch (command) {
            case CommandType.SIGN_IN:
                //签入命令，将连接打上站的标记, 并计入redis, 代表此站在这台服务器上{"stationConn:st001": "serverId/channelId"}, 12s超时
                byte[] stationCodeBytes = ArrayUtils.subarray(data, 0, 8);
                stationCode = BytesUtil.parseString(stationCodeBytes);
                log.info("SIGN_IN:" + stationCode);
                stationConnInfo = new StationConnInfo();
                stationConnInfo.setServerInstanceId(serverConfig.getInstanceId());
                stationConnInfo.setConnIdStr(StationCenter.getChannelIdStr(ctx.channel()));

                stationRegisterkeys = new ArrayList();
                stationRegisterkeys.add("StationConn:" + stationCode);
                log.info("sign in stationCode:{}", stationCode);
                registerSuccess = (Long) redisTemplate.execute(stationRegisterScript, stationRegisterkeys, stationConnInfo, 15);
                StationCenter.markStation(ctx.channel(), stationCode);
                stationMessageReply = new StationMessage();
                stationMessageReply.setCommand(CommandType.SIGN_IN_REPLY);
                replyBytes = new byte[9];
                BytesUtil.setSubBytes(replyBytes, 0, stationCodeBytes);
                log.info("sign in result:{}", registerSuccess);
                if (registerSuccess.equals(0L)) {
                    replyBytes[8] = (byte) 0;
                    stationMessageReply.setData(replyBytes);
                    ctx.channel().writeAndFlush(stationMessageReply);
                } else {
                    //注册失败，已经有这个站了
                    replyBytes[8] = (byte) 1;
                    stationMessageReply.setData(replyBytes);
                    ctx.channel().writeAndFlush(stationMessageReply);
                    ctx.channel().close();
                }
                break;
            case CommandType.HEART_BEAT:
                //心跳命令，计入redis，代表此站在这台服务器上{"stationConn:st001": "serverId/channelId"}，12s超时
                stationCode = (String) Optional.ofNullable(ctx.channel().attr(AttributeKey.valueOf("station")).get()).orElse("");
                if (StringUtils.isEmpty(stationCode)) {
                    break;
                }
                stationConnInfo = new StationConnInfo();
                stationConnInfo.setServerInstanceId(serverConfig.getInstanceId());
                stationConnInfo.setConnIdStr(StationCenter.getChannelIdStr(ctx.channel()));

                stationRegisterkeys = new ArrayList();
                stationRegisterkeys.add("StationConn:" + stationCode);
                registerSuccess = (Long) redisTemplate.execute(stationRegisterScript, stationRegisterkeys, stationConnInfo, 15);
//                stationMessageReply = new StationMessage();
                if (registerSuccess.equals(0l)) {
                    //回复成功
                    stationMessageReply = new StationMessage();
                    stationMessageReply.setCommand(404);
                    stationMessageReply.setData(new byte[1]);
                } else {
                    //注册失败，有可能已经有站注册了,一个站只能有一个连接，关闭
                    log.info("心跳失败,stationCode:{}", stationCode);
                    ctx.channel().close();
                }
                break;
            case CommandType.EXCHANGE_EVENT:
                stationCode = (String) Optional.ofNullable(ctx.channel().attr(AttributeKey.valueOf("station")).get()).orElse("");
//                stationCode = "station_001";
                if (StringUtils.isEmpty(stationCode)) {
                    break;
                }
                //为了削峰，将对象构建后，放到kafka中，由业务中心处理。
                ExchangeEvent exchangeEvent = new ExchangeEvent(stationCode, data);
                //log.info("exchangeEvent:" + exchangeEvent);
                topic = serverConfig.getServiceCenterName() + "_ExchangeEvent";
                //log.info("send to {}", topic);
                kafkaTemplate.send(topic, JSON.toJSONString(exchangeEvent));
                break;
            case CommandType.CHARGER_STATE:
                stationCode = (String) Optional.ofNullable(ctx.channel().attr(AttributeKey.valueOf("station")).get()).orElse("");
//                stationCode = "station_001";
                if (StringUtils.isEmpty(stationCode)) {
                    break;
                }
                ChargerState chargerState = new ChargerState(stationCode, data);
                log.info("chargerState:" + chargerState);
                topic = serverConfig.getServiceCenterName() + "_ChargerState";
                log.info("send to {}", topic);
                kafkaTemplate.send(topic, JSON.toJSONString(chargerState));
                byte[] chargerIdBytes = ArrayUtils.subarray(data, 0, 1);
                byte[] chargerCodeBytes = ArrayUtils.subarray(data, 1, 33);
                byte[] gunNoBytes = ArrayUtils.subarray(data, 33, 34);
                byte[] timeBytes = ArrayUtils.subarray(data, 184, 192);
                stationMessageReply = new StationMessage();
                stationMessageReply.setCommand(CommandType.CHARGER_STATE_REPLY);
                replyBytes = new byte[43];
                BytesUtil.setSubBytes(replyBytes,0,chargerIdBytes);
                BytesUtil.setSubBytes(replyBytes,1,chargerCodeBytes);
                BytesUtil.setSubBytes(replyBytes,33,gunNoBytes);
                BytesUtil.setSubBytes(replyBytes,34,timeBytes);
                replyBytes[42] = (byte) 0;
                ctx.writeAndFlush(replyBytes);
                break;
            case CommandType.REPLY_EXCHANGE:
                stationCode = (String) Optional.ofNullable(ctx.channel().attr(AttributeKey.valueOf("station")).get()).orElse("");
                if (StringUtils.isEmpty(stationCode)) {
                    break;
                }
                ExchangeStartReply startSwapReply = new ExchangeStartReply(stationCode, data);
                topic = serverConfig.getServiceCenterName() + "_ExchangeStartReply";
                log.info("启动换电回复：{}",startSwapReply);
                kafkaTemplate.send(topic, JSON.toJSONString(startSwapReply));
                break;
            case CommandType.EXCHANGE_RECORD:
                stationCode = (String) Optional.ofNullable(ctx.channel().attr(AttributeKey.valueOf("station")).get()).orElse("");
                if (StringUtils.isEmpty(stationCode)) {
                    break;
                }
                //log.info("上传换电记录：{}",data);
                ExchangeRecordInfo exchangeRecordInfo = new ExchangeRecordInfo(stationCode,data);
                topic = serverConfig.getServiceCenterName() + "_UploadExchangeRecord";
                log.info("换电记录上传：{}",exchangeRecordInfo);
                kafkaTemplate.send(topic, JSON.toJSONString(exchangeRecordInfo));
                stationMessageReply = new StationMessage();
                stationMessageReply.setCommand(CommandType.EXCHANGE_RECORD_REPLY);
                replyBytes = new byte[65];
                BytesUtil.setSubBytes(replyBytes,0, BytesUtil.fromString(exchangeRecordInfo.getExchangeNo(),32,(byte)0));
                BytesUtil.setSubBytes(replyBytes,32, BytesUtil.fromString(exchangeRecordInfo.getSerialNo(),32,(byte)0));
                replyBytes[64] = (byte) 0;
                stationMessageReply.setData(replyBytes);
                ctx.writeAndFlush(stationMessageReply);
                break;
            default:
                break;
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        super.exceptionCaught(ctx, cause);
        cause.printStackTrace();
        log.error(cause.getMessage());
    }
}
